;	######################################
;     Refilling shroom (made for the F.L.U.D.D.)
;	           by WhiteYoshiEgg
;	######################################


;######################################
;		F.A.Q.
;######################################


;	* What's this?
;		This sprite is intended to be used together with the F.L.U.D.D., which should
;		be included in this folder. It's a blue mushroom that vanishes when you touch it
;		and instantly refills your water counter.

;	* Anything else I should know?
;		If you've changed the defines in the FLUDD's ASM file, make sure to change them
;		here too, or the sprite will have the wrong effect (or none at all).

;	Refer to the FAQ in the FLUDD's ASM file for more information.






;######################################
;		Defines
;######################################

		!Water		= $0670	;   Free RAM address, used for water counter
		!MaxWater	= 25	;   Max. amount of water (yes, it's decimal)
		!SFX		= $02	;   What sound to play when you touch the sprite


;######################################
;		INIT Routine
;######################################

print "INIT ",pc
		RTL



;######################################
;		MAIN Routine
;######################################

print "MAIN ",pc
		
		PHB : PHK : PLB : JSR SpriteCode : PLB : RTL


	SpriteCode:
		
		JSR SubGFX		;    Graphics

		JSL $01802A		;    Add gravity (in case the sprite is not on the ground)

		JSL $01A7DC		; \  Do nothing when
		BCC Return		; /  not touched

		JSR SubSmoke		;    Generate smoke
		LDA #!MaxWater		; \  Give max. amount
		STA !Water		; /  of water
		LDA #!SFX		; \  Play a sound
		STA $1DFC		; /
		STZ $14C8,x		;    This is the end.

	Return:
		RTS			;    So is this.
		



	SubSmoke:			; \ find a free slot to display effect
		LDY #$03		;  |
	FindFree:			;  |
		LDA $17C0,y		;  |
		BEQ FoundOne		;  |
		DEY			;  |
		BPL FindFree		;  |
		RTS			; / return if no slots open

	FoundOne:
		LDA #$01		; \ set effect graphic to smoke graphic
		STA $17C0,y		; /
		LDA $01			; \ smoke y position = generator y position
		STA $17C4,y		; /
		LDA #$18		; \ set time to show smoke
		STA $17CC,y		; /
		LDA $00			; \ load generator x position and store it for later
		STA $17C8,y		; /
		LDX $15E9
             
		RTS



	SubGFX:
		JSR GetDrawInfo		; after: Y = index to sprite OAM ($300)
					;      $00 = sprite x position relative to screen border 
					;      $01 = sprite y position relative to screen border  
                    
		LDA $157C,x		; $02 = sprite direction
		STA $02

		; if you wish to draw more than one tile, each step between the lines must be repeated
		;*************************************************************************************  

		LDA $00			; set x position of the tile
		STA $0300,y

		LDA $01			; set y position of the tile
		STA $0301,y

		LDA #$24		; set tile number
		STA $0302,y

		LDA $15F6,x		; get sprite palette info
		PHX
		LDX $02			; flip the tile if sprite direction is 0
		BNE +
		ORA #$40
	+           
		PLX
		ORA $64			; add in the priority bits from the level settings
		STA $0303,y		; set properties

		;INY #4			; get the index to the next slot of the OAM (not needed here?)

                    
		;***********************************************************************************

                
		LDY #$02		; #$02 means the tiles are 16x16
		LDA #$00		; This means we drew 1 tile
		JSL $01B7B3
		RTS



	SprT1:	db $0C,$1C
	SprT2:	db $01,$02

	GetDrawInfo:
		STZ $186C,x		; reset sprite offscreen flag, vertical
		STZ $15A0,x		; reset sprite offscreen flag, horizontal
		LDA $E4,x		; \
		CMP $1A			;  | set horizontal offscreen if necessary
		LDA $14E0,x		;  |
		SBC $1B			;  |
		BEQ OnScreenX:		;  |
		INC $15A0,x		; /
	OnScreenX:
		LDA $14E0,x		; \
                XBA			;  |
                LDA $E4,x		;  |
                REP #$20		;  |
		SEC			;  |
                SBC $1A			;  | mark sprite invalid if far enough off screen
                CLC			;  |
                ADC.w #$004		;  |
                CMP.w #$0180		;  |
                SEP #$20		;  |
                ROL A			;  |
                AND #$01		;  |
                STA $15C4,x		;  | 
                BNE Invalid 		; /
                
                LDY #$00		; \ set up loop:
                LDA $1662,x		;  | 
                AND #$20		;  | if not smushed (1662 & 0x20), go through loop twice
                BEQ OnScreenLoop	;  | else, go through loop once
                INY			; / 
	OnScreenLoop:
		LDA $D8,x		; \ 
		CLC			;  | set vertical offscreen if necessary
		ADC SprT1,y		;  |
		PHP			;  |
		CMP $1C			;  | (vert screen boundary)
		ROL $00			;  |
		PLP			;  |
		LDA $14D4,x		;  | 
		ADC #$00		;  |
		LSR $00			;  |
		SBC $1D			;  |
		BEQ OnScreenY		;  |
		LDA $186C,x		;  | (vert offscreen)
		ORA SprT2,y		;  |
		STA $186C,x		;  |
	OnScreenY:			;  |
		DEY			;  |
		BPL OnScreenLoop	; /

		LDY $15EA,x		;    get offset to sprite OAM
		LDA $E4,x		; \ 
		SEC			;  | 
		SBC $1A			;  | $00 = sprite x position relative to screen boarder
		STA $00			; / 
		LDA $D8,x		; \ 
		SEC			;  | 
		SBC $1C			;  | $01 = sprite y position relative to screen boarder
		STA $01			; / 
		RTS			;    return

	Invalid:
		PLA			; \  return from *main gfx routine* subroutine...
		PLA			;  | ...(not just this subroutine)
		RTS			; /		